#!/usr/bin/env python3

' gether os statistics to server.'

import fcntl
import json
import os
import platform
import requests
import socket
import sys
import syslog
import uuid
import time
import threading
import traceback

class pLock(object):
    def __init__(self, lockfile):
        self.lockfile = lockfile
        self.fd = os.open(lockfile, os.O_RDWR | os.O_CREAT)
 
    def lock(self):
        try:
            fcntl.lockf(self.fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
            return True
        except:
            self.fd.close()
            return False

    def unlock(self):
        fcntl.lockf(self.fd, fcntl.LOCK_UN)

    def __del__(self):
        os.close(self.fd)
        os.unlink(self.lockfile)

################################################################################
MAX_TRY = 5
try_count = 1

def get_host_ip():
    ip = None
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip 
        
def get_mac_address(): 
    mac = uuid.UUID(int = uuid.getnode()).hex[-12:] 
    return ":".join([mac[e:e+2] for e in range(0,11,2)])
    
def get_uuid():    
    os_uuid = None
    uuid_file = '/etc/.statistic_os_uuid'
    if not os.path.isfile(uuid_file):
        os_uuid = str(uuid.uuid1())         
        with open(uuid_file, 'w') as f:                    
            f.write(os_uuid)
    else:
        with open(uuid_file, 'r') as f:
            os_uuid = f.read()
    return os_uuid
    
def get_os_release():
    return platform.linux_distribution()[0] + platform.linux_distribution()[1]

def statistic():
    global try_count
    # print("###%d" % try_count) 
    try:
        post_data = {}
        post_data['ip'] = get_host_ip()
        # post_data['mac_addr'] = get_mac_address()
        post_data['uuid'] = get_uuid()
        post_data['os'] = get_os_release()
        url = 'http://124.16.141.197:83/mac/'
        # print(json.dumps(post_data))
        requests.post(url, json.dumps(post_data))
    except:               
        if try_count < MAX_TRY:
            delay = power(2, try_count) * 60
            timer = threading.Timer(delay, statistic)                   
            timer.start()
        try_count = try_count + 1
    
def main():
    # Don't run it in livecd.
    with open('/proc/cmdline', 'r') as f:
        if 'casper' in f.read():
            sys.exit(-1)

    lock = pLock('/var/lock/nfs-statistic.lock');
    if (not lock.lock()):
        syslog.syslog('nfs-statistic acquire lock failed!');
        sys.exit(-1)
    try:
        statistic()
    except:        
        syslog.syslog('nfs-statistic exception:%s' % traceback.format_exc());
    finally:
        lock.unlock()   

################################################################################
if __name__=='__main__':
    main()
